"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const path_1 = require("path");
const root_command_1 = tslib_1.__importDefault(require("../../root-command"));
const compile = tslib_1.__importStar(require("../../utils/compiler"));
const error_1 = require("../../utils/error");
const compile_1 = tslib_1.__importDefault(require("../service/compile"));
class ProcessCompile extends root_command_1.default {
    async run() {
        const { args, flags } = this.parse(ProcessCompile);
        const definition = await compile.process(fs_1.readFileSync(args.PROCESS_FILE), async (instanceObject) => {
            if (instanceObject.instanceHash) {
                return instanceObject.instanceHash;
            }
            if (!flags.dev) {
                throw new Error('"instanceHash" should be present in your process. Use `--dev` to be able to use the "instance" attributes.');
            }
            if (!instanceObject.instance) {
                throw new Error('"instanceHash" or "instance" not found in the process\'s definition');
            }
            const { service, src, env } = instanceObject.instance;
            if (service) {
                return this.serviceToInstance(service, env);
            }
            if (src) {
                return this.sourceToInstance(args.PROCESS_FILE, src, env);
            }
            throw new Error('at least one of the following parameter should be set: "instanceHash", "service" or "src"');
        }, (flags.env || []).reduce((prev, env) => (Object.assign({}, prev, { [env.split('=')[0]]: env.split('=')[1] })), {}));
        this.styledJSON(definition);
        this.spinner.stop();
        return definition;
    }
    async sourceToInstance(dir, src, env) {
        const directory = path_1.join(path_1.dirname(dir), src);
        const definition = await compile_1.default.run([fs_1.existsSync(directory) ? directory : src, '--silent']);
        let hash;
        try {
            const resp = await this.api.service.create(definition);
            if (!resp.hash)
                throw new Error('invalid hash');
            hash = resp.hash;
        }
        catch (e) {
            if (!error_1.IsAlreadyExistsError.match(e))
                throw e;
            hash = new error_1.IsAlreadyExistsError(e).hash;
        }
        if (!hash)
            throw new Error('hash cannot be empty');
        return this.serviceToInstance(hash, env);
    }
    async serviceToInstance(sidOrHash, env) {
        const { services } = await this.api.service.list({});
        if (!services)
            throw new Error('no services deployed, please deploy your service first');
        const sidOrHashStr = sidOrHash.toString();
        const match = services.filter(x => x.hash && x.hash.toString() === sidOrHashStr || x.sid && x.sid.toString() === sidOrHashStr);
        if (!match || match.length === 0)
            throw new Error(`cannot find any service with the following: ${sidOrHashStr}`);
        if (match.length > 1)
            throw new Error(`multiple services match the following sid: ${sidOrHashStr}, provide a service's hash instead`);
        const service = match[0];
        if (!service.hash)
            throw new Error('invalid service');
        try {
            const resp = await this.api.instance.create({
                serviceHash: service.hash,
                env,
            });
            if (!resp.hash)
                throw new Error('invalid hash');
            return resp.hash;
        }
        catch (e) {
            if (!error_1.IsAlreadyExistsError.match(e))
                throw e;
            return new error_1.IsAlreadyExistsError(e).hash;
        }
    }
}
ProcessCompile.description = 'Compile a process';
ProcessCompile.flags = Object.assign({}, root_command_1.default.flags, { dev: command_1.flags.boolean({ description: 'compile the process and automatically deploy and start all the services' }), env: command_1.flags.string({
        description: 'Set environment variables',
        multiple: true,
        helpValue: 'FOO=BAR'
    }) });
ProcessCompile.args = [{
        name: 'PROCESS_FILE',
        description: 'Path of a process file'
    }];
exports.default = ProcessCompile;
