"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path_1 = require("path");
const account_command_1 = require("../../account-command");
const marketplace_command_1 = tslib_1.__importDefault(require("../../marketplace-command"));
const root_command_1 = tslib_1.__importDefault(require("../../root-command"));
const compile = tslib_1.__importStar(require("../../utils/compiler"));
const deployer_1 = tslib_1.__importStar(require("../../utils/deployer"));
const ipfsClient = require('ipfs-http-client');
class ServiceCompile extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.IPFS = ipfsClient('ipfs.app.mesg.com', '5001', { protocol: 'http' });
    }
    async run() {
        const { args } = this.parse(ServiceCompile);
        this.spinner.status = 'Download sources';
        const path = await deployer_1.default(await this.processUrl(args.SERVICE));
        const definition = await compile.service(fs_1.readFileSync(path_1.join(path, 'mesg.yml')));
        definition.source = await this.deploySources(path);
        this.styledJSON(definition);
        this.spinner.stop();
        return definition;
    }
    async getAuthorizedServiceInfo(id, versionHash) {
        const { addresses } = await this.execute({
            instanceHash: await this.engineServiceInstance(account_command_1.WithoutPassphrase.SERVICE_NAME),
            taskKey: 'list'
        });
        if (!addresses.length) {
            throw new Error('no available account. please add an authorized account in order to deploy the service');
        }
        const { authorized, sid, source, type } = await this.execute({
            instanceHash: await this.engineServiceInstance(marketplace_command_1.default.SERVICE_NAME),
            taskKey: 'isAuthorized',
            inputs: {
                id,
                versionHash,
                addresses
            }
        });
        if (!authorized) {
            throw new Error('no available account. please add an authorized account in order to deploy the service');
        }
        return { sid, source, type };
    }
    async processUrl(url) {
        const marketplaceUrl = url.split('mesg://marketplace/service/');
        if (marketplaceUrl.length === 2) {
            const versionHash = marketplaceUrl[1];
            const { type, source } = await this.getAuthorizedServiceInfo('', versionHash);
            if (type === 'ipfs') {
                return `http://ipfs.app.mesg.com:8080/ipfs/${source}`; // tslint:disable-line:no-http-string
            }
            throw new Error(`unknown protocol '${type}'`);
        }
        return url;
    }
    async deploySources(path) {
        const buffer = [];
        return new Promise((resolve, reject) => {
            deployer_1.createTar(path_1.join(path))
                .on('data', (data) => buffer.push(...data))
                .once('error', reject)
                .on('end', async () => resolve(await this.upload(Buffer.from(buffer))));
        });
    }
    async upload(buffer) {
        const res = await this.IPFS.add(Buffer.from(buffer), { pin: true });
        if (!res.length) {
            throw new Error('pushing service manifest failed: run service:compile again');
        }
        return res[0].hash;
    }
}
ServiceCompile.description = 'Compile a service and upload its source to IPFS server';
ServiceCompile.flags = Object.assign({}, root_command_1.default.flags);
ServiceCompile.args = [{
        name: 'SERVICE',
        description: 'Path or url ([https|mesg]://) of a service',
        default: './'
    }];
exports.default = ServiceCompile;
