"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const api_1 = require("mesg-js/lib/api");
const base58 = tslib_1.__importStar(require("mesg-js/lib/util/base58"));
const encoder_1 = require("mesg-js/lib/util/encoder");
const node_docker_api_1 = require("node-docker-api");
const root_command_1 = tslib_1.__importDefault(require("../../root-command"));
const docker_1 = require("../../utils/docker");
const instance_resolver_1 = tslib_1.__importDefault(require("../../utils/instance-resolver"));
class ServiceLogs extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.docker = new node_docker_api_1.Docker(null);
    }
    async run() {
        const { args, flags } = this.parse(ServiceLogs);
        const instanceHash = await instance_resolver_1.default(this.api, args.INSTANCE_HASH);
        const streams = [];
        const dockerServices = await this.docker.service.list({
            filters: {
                label: [`mesg.instance=${base58.encode(instanceHash)}`]
            }
        });
        for (const service of dockerServices) {
            const labels = service.data.Spec.Labels;
            const name = [labels['mesg.sid'], labels['mesg.dependency']].join('/');
            const logs = await service.logs({
                stderr: true,
                stdout: true,
                follow: flags.follow,
                tail: flags.tail && flags.tail >= 0 ? flags.tail : 'all'
            });
            streams.push(() => logs.destroy());
            logs
                .on('data', (buffer) => docker_1.parseLog(buffer).forEach(x => this.log(chalk_1.default.gray(name + ':'), x)))
                .on('error', (error) => { this.warn('Docker log stream error: ' + error.message); });
        }
        if (flags.results) {
            const results = this.api.execution.stream({
                filter: {
                    instanceHash,
                    statuses: [
                        api_1.ExecutionStatus.COMPLETED,
                        api_1.ExecutionStatus.FAILED,
                    ],
                    taskKey: flags.task
                }
            });
            streams.push(() => results.cancel());
            results
                .on('data', data => this.log(this.formatResult(data)))
                .on('error', (error) => { this.warn('Result stream errror: ' + error.message); });
        }
        if (flags.events) {
            const events = this.api.event.stream({
                filter: {
                    instanceHash,
                    key: flags.event,
                }
            });
            streams.push(() => events.cancel());
            events
                .on('data', (data) => this.log(this.formatEvent(data)))
                .on('error', (error) => { this.warn('Event stream error: ' + error.message); });
        }
        return {
            destroy: () => {
                streams.forEach(s => s());
            }
        };
    }
    formatEvent(event) {
        if (!event.data)
            return;
        return `EVENT[${event.key}]: ` + chalk_1.default.gray(JSON.stringify(encoder_1.decode(event.data)));
    }
    formatResult(execution) {
        if (execution.error) {
            return `RESULT[${execution.taskKey}]: ` + chalk_1.default.red('ERROR:', execution.error);
        }
        if (!execution.outputs)
            return;
        return `RESULT[${execution.taskKey}]: ` + chalk_1.default.gray(JSON.stringify(encoder_1.decode(execution.outputs)));
    }
}
ServiceLogs.description = 'Fetch the logs of a service';
ServiceLogs.flags = Object.assign({}, root_command_1.default.flags, { 
    // dependency: flags.string({
    //   char: 'd',
    //   description: 'Name of the dependency to show the logs from',
    //   multiple: true
    // }),
    events: command_1.flags.boolean({
        description: 'Display events',
        allowNo: true,
        default: true
    }), results: command_1.flags.boolean({
        description: 'Display results',
        allowNo: true,
        default: true
    }), event: command_1.flags.string({
        description: 'Display a specific event'
    }), task: command_1.flags.string({
        description: 'Display a specific task results'
    }), tail: command_1.flags.integer({
        description: 'Display the last N lines',
        default: -1
    }), follow: command_1.flags.boolean({
        description: 'Follow log output',
        allowNo: true,
        default: true
    }) });
ServiceLogs.args = [{
        name: 'INSTANCE_HASH',
        required: true,
    }];
exports.default = ServiceLogs;
