"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const mesg_js_1 = require("mesg-js");
const api_1 = tslib_1.__importDefault(require("mesg-js/lib/api"));
const base58 = tslib_1.__importStar(require("mesg-js/lib/util/base58"));
const util_1 = require("util");
class default_1 extends command_1.Command {
    constructor(argv, config) {
        super(argv, config);
        const { flags } = this.parse();
        const port = flags.port;
        const host = process.env.DOCKER_HOST
            ? new URL(process.env.DOCKER_HOST).hostname
            : flags.host;
        const endpoint = `${host}:${port}`;
        this.api = api_1.default(endpoint);
        this._app = mesg_js_1.application({ endpoint });
    }
    get spinner() {
        const { flags } = this.parse();
        const nope = () => { };
        if (flags.quiet) {
            return { start: nope, stop: (message) => message ? this.log(message) : null, status: null };
        }
        if (flags.silent) {
            return { start: nope, stop: nope, status: null };
        }
        return cli_ux_1.cli.action;
    }
    log(message, ...args) {
        if (this.parse) {
            const { flags } = this.parse();
            if (flags.silent) {
                return;
            }
        }
        message = typeof message === 'string' ? message : util_1.inspect(message);
        process.stdout.write(util_1.format(message, ...args) + '\n');
    }
    styledJSON(data) {
        const { flags } = this.parse();
        if (flags.silent)
            return;
        const base58EncodedHash = JSON.parse(JSON.stringify(data, (key, value) => {
            return key && RegExp('hash$', 'i').test(key) && value && value.type === 'Buffer'
                ? base58.encode(value.data)
                : value;
        }));
        cli_ux_1.cli.styledJSON(base58EncodedHash);
    }
    async catch(err) {
        try {
            await super.catch(err);
        }
        catch (e) {
            if (e.message) {
                this.error(err.message);
            }
            else {
                throw e;
            }
        }
    }
    async execute(request) {
        const exec = await this._app.executeTaskAndWaitResult({
            instanceHash: request.instanceHash,
            tags: request.tags || [],
            taskKey: request.taskKey,
            inputs: this._app.encodeData(request.inputs || {})
        });
        if (exec.error)
            throw new Error(exec.error);
        if (!exec.outputs)
            throw new Error('missing outputs');
        return this._app.decodeData(exec.outputs);
    }
    async info() {
        return this.api.core.info();
    }
    async engineServiceInstance(key) {
        const info = await this.info();
        const service = info.services.find((x) => x.key === key);
        if (!service) {
            throw new Error(`Cannot find service ${key}`);
        }
        return service.hash;
    }
}
default_1.flags = {
    help: command_1.flags.help({ char: 'h' }),
    quiet: command_1.flags.boolean({ char: 'q', description: 'Display only essential information' }),
    silent: command_1.flags.boolean({ hidden: true }),
    port: command_1.flags.integer({ char: 'p', default: 50052, description: 'Port to access the MESG engine' }),
    host: command_1.flags.string({ default: 'localhost', description: 'Host to access the MESG engine' })
};
exports.default = default_1;
