"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const js_yaml_1 = tslib_1.__importDefault(require("js-yaml"));
const lodash_pick_1 = tslib_1.__importDefault(require("lodash.pick"));
const replaceVariable = (value, env) => {
    if (typeof value !== 'string')
        return value;
    const reg = new RegExp('\\$\\(env\\:(.*)\\)', 'g');
    return value.replace(reg, (match, p1) => {
        if (!Object.keys(env).includes(p1)) {
            throw new Error('env variable ' + p1 + ' must be set');
        }
        return env[p1];
    });
};
const decode = (content, env) => {
    const data = js_yaml_1.default.safeLoad(content.toString());
    return JSON.parse(JSON.stringify(data), function (key, value) {
        return replaceVariable(value, env);
    });
};
const mapToArray = (inputs) => Object.keys(inputs || {}).map(key => (Object.assign({}, inputs[key], { key })));
const parseParams = (params) => mapToArray(params).map(x => (Object.assign({}, lodash_pick_1.default(x, ['key', 'name', 'description', 'type', 'repeated', 'optional']), { object: parseParams(x.object) })));
exports.service = async (content) => {
    const definition = decode(content, {});
    return Object.assign({}, lodash_pick_1.default(definition, ['sid', 'name', 'description', 'repository']), { configuration: definition.configuration || {}, dependencies: mapToArray(definition.dependencies), tasks: mapToArray(definition.tasks).map(x => (Object.assign({}, lodash_pick_1.default(x, ['key', 'name', 'description']), { inputs: parseParams(x.inputs), outputs: parseParams(x.outputs) }))), events: mapToArray(definition.events).map(x => (Object.assign({}, lodash_pick_1.default(x, ['key', 'name', 'description']), { data: parseParams(x.data) }))) });
};
const nodeCompiler = async (type, def, key, opts) => {
    const nodes = {
        result: async (def, key, opts) => ({
            key,
            taskKey: def.taskKey,
            instanceHash: opts.instanceResolver ? await opts.instanceResolver(def) : def.instanceHash
        }),
        event: async (def, key, opts) => ({
            key,
            eventKey: def.eventKey,
            instanceHash: opts.instanceResolver ? await opts.instanceResolver(def) : def.instanceHash
        }),
        task: async (def, key, opts) => ({
            key,
            taskKey: def.taskKey,
            instanceHash: opts.instanceResolver ? await opts.instanceResolver(def) : def.instanceHash
        }),
        map: async (def, key, opts) => ({
            key,
            outputs: Object.keys(def).map(key => ({
                key,
                ref: {
                    key: def[key].key,
                    nodeKey: def[key].stepKey || opts.defaultNodeKey,
                }
            }))
        }),
        filter: async (def, key, opts) => ({
            key,
            conditions: Object.keys(def.conditions).map(key => ({
                key,
                predicate: 1,
                value: def.conditions[key]
            }))
        })
    };
    return {
        [type]: await nodes[type](def, key, opts)
    };
};
exports.process = async (content, instanceResolver, envs) => {
    const definition = decode(content, envs);
    let nodes = [];
    let edges = [];
    let previousKey = null;
    let previousKeyWithOutputs = null;
    let i = 0;
    for (const step of definition.steps) {
        step.key = step.key || `node-${i}`;
        if (step.inputs) {
            const mapKey = `${step.key}-inputs`;
            const mapNode = await nodeCompiler('map', step.inputs, mapKey, { defaultNodeKey: previousKeyWithOutputs });
            nodes.push(mapNode);
            if (previousKey) {
                edges.push({ src: previousKey, dst: mapKey });
            }
            previousKeyWithOutputs = previousKey = mapKey;
            i++;
        }
        const type = step.type !== 'trigger'
            ? step.type
            : step.eventKey
                ? 'event'
                : 'result';
        const stepNode = await nodeCompiler(type, step, step.key, { instanceResolver });
        nodes.push(stepNode);
        if (previousKey) {
            edges.push({ src: previousKey, dst: step.key });
        }
        if (type !== 'filter') {
            previousKeyWithOutputs = step.key;
        }
        previousKey = step.key;
        i++;
    }
    return {
        key: definition.key,
        nodes,
        edges,
    };
};
